<?php

# execute using following command
# php mr_newcase.php -d database_name -h database_host -u db_user

$shortopts = "d:h:u:p:";
$options = getopt($shortopts);

if (array_key_exists('d', $options)) {
	    $database = $options['d'];
} else {
	    die("no database specified with -d switch\n");
}

if (array_key_exists('h', $options)) {
	    $host = $options['h'];
} else {
	    die("no host specified with -h switch\n");
}

if (array_key_exists('u', $options)) {
	    $user = $options['u'];
} else {
	    die("no user specified with -u switch\n");
}

if (array_key_exists('p', $options)) {
	    $password = $options['p'];
} else {
            echo "Enter password:\n";
            system('stty -echo');
            $password = trim(fgets(STDIN));
            system('stty echo');
}

logger("Host is $host and database is $database");

$mysqli = mysqli_connect($host, $user, $password, $database);
if (mysqli_connect_errno($mysqli)) {
	  echo "Failed to connect to $database: " . mysqli_connect_error();
}
logger("set long timeout value");
$res = mysqli_query($mysqli, 'SET session wait_timeout=1440*60;') or die(mysqli_error($mysqli));
// Drop any residual work tables

logger("dropping new_cases table if it exists");
$res = mysqli_query($mysqli, 'drop table if exists new_cases') or die(mysqli_error($mysqli));

logger("dropping deadwood_cases table if it exists");
$res = mysqli_query($mysqli, 'drop table if exists deadwood_cases') or die(mysqli_error($mysqli));

logger("dropping old_cases table if it exists");
$res = mysqli_query($mysqli, 'drop table if exists old_cases') or die(mysqli_error($mysqli));

logger("counting rows in cases table");
$res = mysqli_query($mysqli, 'select count(id) as cases FROM cases') or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $cases = $row['cases'];
   logger("total number of rows in cases table: $cases");
}

// We need to check the integrity of the cases table before we prune it so we can compare afterwards

$missing_content_ids = 0 ;
logger("Checking original values for missing cases and content ids");
$query = 'select count(content_id) as cases from tests where is_completed = 1 and is_selected = 1 and content_id not in (select id from cases)';
logger($query);
$res = mysqli_query($mysqli, $query) or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $missing_content_ids = $row['cases'];
}

$missing_case_ids = 0 ;
$query = 'select count(case_id) as cases from tests where is_completed = 0 and is_selected = 1 and case_id not in (select id from cases)';
logger($query);
$res = mysqli_query($mysqli, $query) or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $missing_case_ids = $row['cases'];
}
logger("Starting values: " . $missing_content_ids. "\t" . $missing_case_ids );

logger("creating new_cases table");
$res = mysqli_query($mysqli, "create table new_cases like cases") or die(mysqli_error($mysqli));

logger("creating deadwood_cases table");
$res = mysqli_query($mysqli, "create table deadwood_cases like cases") or die(mysqli_error($mysqli));

logger("locking all tables");
$res = mysqli_query($mysqli, 'lock tables cases write, tests read, tests as t2 read, new_cases write, deadwood_cases write') or die(mysqli_error($mysqli));

$query =  "insert into new_cases select * from cases where is_copy = 0 or id in (select case_id as id from tests where is_selected = 1 and is_completed = 0 and case_id is not null) or id in (select content_id as id from tests as t2 where is_selected = 1 and is_completed = 1 and content_id is not null)";
logger("$query");
$res = mysqli_query($mysqli, $query) or die(mysqli_error($mysqli));

$query =  "insert into deadwood_cases select * from cases where id not in (select id from new_cases)";
logger("$query");
$res = mysqli_query($mysqli, $query) or die(mysqli_error($mysqli));

logger("counting rows in new_cases and deadwood_cases tables");
$res = mysqli_query($mysqli, 'select count(id) as cases FROM new_cases') or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $cases = $row['cases'];
   logger("total number of rows in new_cases table: $cases");
}

$res = mysqli_query($mysqli, 'select count(id) as cases FROM deadwood_cases') or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $cases = $row['cases'];
   logger("number of rows in deadwood_cases table: $cases");
}

logger("checking integrity of new_cases table - check 1");
$res = mysqli_query($mysqli, 'select count(content_id) as cases from tests where is_completed = 1 and is_selected = 1 and content_id not in (select id from new_cases)') or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $cases = $row['cases'];
}
logger("Number of missing content ids: " . $cases);
if ($cases != $missing_content_ids) {die("Integrity failure!\n");}

logger("checking integrity of new_cases table - check 2");
$res = mysqli_query($mysqli, 'select count(case_id) as cases from tests where is_completed = 0 and is_selected = 1 and case_id not in (select id from new_cases)') or die(mysqli_error($mysqli));
if (mysqli_num_rows($res) > 0) {
   $row = mysqli_fetch_assoc($res);
   $cases = $row['cases'];
}
logger("Number of missing case ids: " . $cases);
if ($cases != $missing_case_ids) {die("Integrity failure!\n");}

logger("renaming tables");

$res = mysqli_query($mysqli, 'alter table cases rename to old_cases') or die(mysqli_error($mysqli));
$res = mysqli_query($mysqli, 'alter table new_cases rename to cases') or die(mysqli_error($mysqli));

logger("unlocking all tables");
$res = mysqli_query($mysqli, 'unlock tables;') or die(mysqli_error($mysqli));

logger("dropping old_cases table");
$res = mysqli_query($mysqli, 'drop table old_cases') or die(mysqli_error($mysqli));

logger("finished");

exit ;

function logger($msg) {
  $date = date('m/d/Y h:i:s a', time());
  echo "$date: $msg\n";
}
